<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

function khattam_log($message, $type = 'info') {
    if (defined('WP_DEBUG') && WP_DEBUG) {
        if (is_array($message) || is_object($message)) {
            $message = wp_json_encode($message);
        }
        
        if (defined('WP_DEBUG_LOG') && WP_DEBUG_LOG) {
            error_log("Khattam [$type]: " . $message);
        }
    }
}

function khattam_add_watermark($file, $custom_position = null, $custom_size = null, $custom_opacity = null) {
    khattam_log(__('Starting watermark application', 'khattam'));
    
    // Save a copy of the original image before processing
    $original_file = get_post_meta($file['attachment_id'], '_khattam_original_file', true);
    
    if (!$original_file || !file_exists($original_file)) {
        $original_file = wp_normalize_path(pathinfo($file['file'], PATHINFO_DIRNAME) . '/' . pathinfo($file['file'], PATHINFO_FILENAME) . '_original.' . pathinfo($file['file'], PATHINFO_EXTENSION));
        if (!file_exists($original_file)) {
            if (!copy($file['file'], $original_file)) {
                khattam_log(__('Failed to save original file', 'khattam'));
                return array('watermarked' => false);
            }
        }
        // Update image metadata
        update_post_meta($file['attachment_id'], '_khattam_original_file', $original_file);
        /* translators: %s: Path to the original saved file */
        khattam_log(sprintf(__('Original file saved at: %s', 'khattam'), $original_file));
    }
    
    // Check for image processing libraries
    if (!extension_loaded('gd') && !extension_loaded('imagick')) {
        khattam_log(__('No image processing library available (requires GD or ImageMagick)', 'khattam'));
        return array('watermarked' => false);
    }
    
    $image_type = wp_check_filetype($file['file']);
    $supported_types = array('image/jpeg', 'image/png', 'image/gif', 'image/webp');
    
    if (!in_array($image_type['type'], $supported_types)) {
        /* translators: %s: File type of the image */
        khattam_log(sprintf(__('Unsupported file type: %s', 'khattam'), $image_type['type']));
        return array('watermarked' => false);
    }
    
    // Check if file exists
    if (!file_exists($file['file'])) {
        /* translators: %s: Path to the source image file */
        khattam_log(sprintf(__('Source image file not found: %s', 'khattam'), $file['file']));
        return array('watermarked' => false);
    }
    
    $logo_path = get_option('khattam_logo');
    if (!$logo_path) {
        khattam_log(__('Logo path not set', 'khattam'));
        return array('watermarked' => false);
    }
    
    // Convert URL to file path
    $upload_dir = wp_upload_dir();
    $logo_path = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $logo_path);
    
    // Ensure correct path
    $logo_path = wp_normalize_path($logo_path);
    
    if (!file_exists($logo_path)) {
        /* translators: %s: Path to the logo file */
        khattam_log(sprintf(__('Logo file not found at: %s', 'khattam'), $logo_path));
        return array('watermarked' => false);
    }
    
    // Use GD directly
    try {
        $source_image = null;
        $logo_image = null;
        
        // Load original image with transparency support
        switch ($image_type['type']) {
            case 'image/jpeg':
                $source_image = imagecreatefromjpeg($file['file']);
                break;
            case 'image/png':
                $source_image = imagecreatefrompng($file['file']);
                imagealphablending($source_image, false);
                imagesavealpha($source_image, true);
                break;
            case 'image/gif':
                $source_image = imagecreatefromgif($file['file']);
                break;
            case 'image/webp':
                $source_image = imagecreatefromwebp($file['file']);
                break;
        }
        
        if (!$source_image) {
            khattam_log(__('Failed to load source image', 'khattam'));
            return array('watermarked' => false);
        }
        
        // Load logo with transparency support
        $logo_type = wp_check_filetype($logo_path);
        switch ($logo_type['type']) {
            case 'image/jpeg':
                $logo_image = imagecreatefromjpeg($logo_path);
                break;
            case 'image/png':
                $logo_image = imagecreatefrompng($logo_path);
                imagealphablending($logo_image, false);
                imagesavealpha($logo_image, true);
                break;
            case 'image/gif':
                $logo_image = imagecreatefromgif($logo_path);
                break;
        }
        
        if (!$logo_image) {
            khattam_log(__('Failed to load logo image', 'khattam'));
            imagedestroy($source_image);
            return array('watermarked' => false);
        }
        
        // Get image dimensions
        $source_width = imagesx($source_image);
        $source_height = imagesy($source_image);
        $logo_width = imagesx($logo_image);
        $logo_height = imagesy($logo_image);
        
        // Use custom values if provided, otherwise use defaults
        $logo_position = $custom_position !== null ? $custom_position : get_option('khattam_logo_position', 'bottom-right');
        $logo_size = $custom_size !== null ? $custom_size : intval(get_option('khattam_logo_size', 10));
        $logo_opacity = $custom_opacity !== null ? $custom_opacity : intval(get_option('khattam_logo_opacity', 100));
        
        // Ensure size and opacity have valid values
        $logo_size = max(1, min(100, $logo_size)); // between 1 and 100
        $logo_opacity = max(0, min(100, $logo_opacity)); // between 0 and 100
        
        // Calculate new logo size
        $new_logo_width = max(1, round(($source_width * $logo_size) / 100));
        $new_logo_height = max(1, round(($logo_height * $new_logo_width) / $logo_width));
        
        // Create resized logo with transparency
        $resized_logo = imagecreatetruecolor($new_logo_width, $new_logo_height);
        imagealphablending($resized_logo, false);
        imagesavealpha($resized_logo, true);
        $transparent = imagecolorallocatealpha($resized_logo, 255, 255, 255, 127);
        imagefilledrectangle($resized_logo, 0, 0, $new_logo_width, $new_logo_height, $transparent);
        imagecopyresampled($resized_logo, $logo_image, 0, 0, 0, 0, $new_logo_width, $new_logo_height, $logo_width, $logo_height);
        
        // Apply opacity to logo
        $opacity = $logo_opacity / 100;
        for ($x = 0; $x < $new_logo_width; $x++) {
            for ($y = 0; $y < $new_logo_height; $y++) {
                $color = imagecolorat($resized_logo, $x, $y);
                $alpha = 127 - (($color >> 24) & 0xFF);
                $alpha = $alpha * $opacity;
                $new_alpha = 127 - $alpha;
                $new_color = imagecolorallocatealpha(
                    $resized_logo,
                    ($color >> 16) & 0xFF,
                    ($color >> 8) & 0xFF,
                    $color & 0xFF,
                    $new_alpha
                );
                imagesetpixel($resized_logo, $x, $y, $new_color);
            }
        }
        
        // Determine logo position
        switch ($logo_position) {
            case 'top-left':
                $x = 0;
                $y = 0;
                break;
            case 'top-right':
                $x = $source_width - $new_logo_width;
                $y = 0;
                break;
            case 'bottom-left':
                $x = 0;
                $y = $source_height - $new_logo_height;
                break;
            case 'bottom-right':
            default:
                $x = $source_width - $new_logo_width;
                $y = $source_height - $new_logo_height;
                break;
        }
        
        // Merge images with transparency
        imagealphablending($source_image, true);
        imagecopy($source_image, $resized_logo, $x, $y, 0, 0, $new_logo_width, $new_logo_height);
        
        // Save image with transparency
        switch ($image_type['type']) {
            case 'image/jpeg':
                imagejpeg($source_image, $file['file'], 90);
                break;
            case 'image/png':
                imagepng($source_image, $file['file'], 9);
                break;
            case 'image/gif':
                imagegif($source_image, $file['file']);
                break;
            case 'image/webp':
                imagewebp($source_image, $file['file'], 80);
                break;
        }
        
        // Clean up memory
        imagedestroy($source_image);
        imagedestroy($logo_image);
        imagedestroy($resized_logo);
        
        return array(
            'file' => $file['file'],
            'watermarked' => true,
            'original_file' => $original_file
        );
        
    } catch (Exception $e) {
        /* translators: %s: Error message from the exception */
        khattam_log(sprintf(__('Error processing image: %s', 'khattam'), $e->getMessage()));
        return array('watermarked' => false);
    }
}

remove_filter('wp_handle_upload', 'khattam_add_watermark');

function khattam_handle_upload_watermark($file) {
    if (get_option('khattam_enabled')) {
        // Add watermark after upload
        add_action('add_attachment', 'khattam_apply_watermark_after_upload');
    }
    return $file;
}
add_filter('wp_handle_upload', 'khattam_handle_upload_watermark');

function khattam_apply_watermark_after_upload($attachment_id) {
    // Ensure uploaded file is an image
    if (!wp_attachment_is_image($attachment_id)) {
        return;
    }

    $file = get_attached_file($attachment_id);
    $file_array = array('file' => $file, 'attachment_id' => $attachment_id);

    // Get default settings
    $logo_position = get_option('khattam_logo_position', 'bottom-right');
    $logo_size = intval(get_option('khattam_logo_size', 10));
    $logo_opacity = intval(get_option('khattam_logo_opacity', 100));

    // Apply watermark
    $result = khattam_add_watermark($file_array, $logo_position, $logo_size, $logo_opacity);

    if ($result['watermarked']) {
        // Update image metadata
        wp_update_attachment_metadata($attachment_id, wp_generate_attachment_metadata($attachment_id, $file));
    }

    // Remove action after execution to avoid duplication
    remove_action('add_attachment', 'khattam_apply_watermark_after_upload');
}