<?php
/**
 * Plugin Name: Khattam
 * Description: Add logo watermark to images.
 * Version: 1.1
 * Author: Ali Malas
 * Author URI: https://mostaql.com/u/Ali__Malas
 * License: GPL v2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: khattam
 * Domain Path: /languages
 */

// Prevent direct access
if (!defined('ABSPATH')) {
    exit;
}

// Add translation support
function khattam_load_textdomain() {
    load_plugin_textdomain('khattam', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
add_action('init', 'khattam_load_textdomain');

// Include necessary files
include_once plugin_dir_path(__FILE__) . 'admin/settings-page.php';
include_once plugin_dir_path(__FILE__) . 'includes/image-watermark.php';
include_once plugin_dir_path(__FILE__) . 'includes/enqueue-scripts.php';

// Ensure script translations
function khattam_set_script_translations() {
    wp_set_script_translations('khattam-admin-script', 'khattam', plugin_dir_path(__FILE__) . 'languages');
    wp_set_script_translations('khattam-watermark-dialog', 'khattam', plugin_dir_path(__FILE__) . 'languages');
    wp_set_script_translations('khattam-settings-page-script', 'khattam', plugin_dir_path(__FILE__) . 'languages');
}
add_action('init', 'khattam_set_script_translations');

function khattam_ajax_apply_watermark() {
    check_ajax_referer('khattam_watermark_nonce', 'nonce');
    
    if (!current_user_can('upload_files')) {
        wp_send_json_error(esc_html__('Unauthorized', 'khattam'));
    }
    
    // Validate and sanitize image_id
    if (!isset($_POST['image_id'])) {
        wp_send_json_error(esc_html__('Image ID is required', 'khattam'));
    }
    $image_id = intval(wp_unslash($_POST['image_id']));
    
    $file_path = get_attached_file($image_id);
    
    if (!$file_path) {
        wp_send_json_error(esc_html__('Image not found', 'khattam'));
    }
    
    // Use values from dialog or default settings
    $logo_position = isset($_POST['position']) ? sanitize_text_field(wp_unslash($_POST['position'])) : get_option('khattam_logo_position', 'bottom-right');
    $logo_size = isset($_POST['size']) ? intval($_POST['size']) : intval(get_option('khattam_logo_size', 10));
    $logo_opacity = isset($_POST['opacity']) ? intval($_POST['opacity']) : intval(get_option('khattam_logo_opacity', 100));
    
    $file = array('file' => $file_path, 'attachment_id' => $image_id);
    $result = khattam_add_watermark($file, $logo_position, $logo_size, $logo_opacity);
    
    if ($result['watermarked']) {
        wp_update_attachment_metadata($image_id, wp_generate_attachment_metadata($image_id, $file_path));
        wp_send_json_success(array('url' => wp_get_attachment_url($image_id)));
    } else {
        wp_send_json_error(esc_html__('Failed to apply watermark', 'khattam'));
    }
}
add_action('wp_ajax_khattam_apply_watermark', 'khattam_ajax_apply_watermark');

// Add watermark button to image details page
function khattam_add_watermark_button($form_fields, $post) {
    // Check if button display option is enabled
    if (get_option('khattam_show_media_buttons', '1') !== '1') {
        return $form_fields;
    }

    // Check file type
    if (!wp_attachment_is_image($post->ID)) {
        return $form_fields;
    }
    
    $logo_position = get_option('khattam_logo_position', 'bottom-right');
    $logo_size = get_option('khattam_logo_size', 10);
    $logo_opacity = get_option('khattam_logo_opacity', 100);
    
    $position_options = array(
        'top-left' => esc_html__('Top Left', 'khattam'),
        'top-right' => esc_html__('Top Right', 'khattam'),
        'bottom-left' => esc_html__('Bottom Left', 'khattam'),
        'bottom-right' => esc_html__('Bottom Right', 'khattam')
    );
    
    $position_select = '<select name="attachments[' . $post->ID . '][khattam_logo_position]" id="attachments-' . $post->ID . '-khattam_logo_position">';
    foreach ($position_options as $value => $label) {
        $position_select .= '<option value="' . esc_attr($value) . '" ' . selected($logo_position, $value, false) . '>' . esc_html($label) . '</option>';
    }
    $position_select .= '</select>';
    
    $size_input = '<input type="number" name="attachments[' . $post->ID . '][khattam_logo_size]" id="attachments-' . $post->ID . '-khattam_logo_size" value="' . esc_attr($logo_size) . '" min="1" max="100" style="width: 60px;">';
    
    $opacity_input = '<input type="number" name="attachments[' . $post->ID . '][khattam_logo_opacity]" id="attachments-' . $post->ID . '-khattam_logo_opacity" value="' . esc_attr($logo_opacity) . '" min="0" max="100" style="width: 60px;">';
    
    $form_fields['khattam_watermark'] = array(
        'label' => esc_html__('Watermark', 'khattam'),
        'input' => 'html',
        'html' => '<div style="margin-bottom: 10px;">
                    <label style="display: block; margin-bottom: 5px;">' . esc_html__('Logo Position:', 'khattam') . '</label>
                    ' . $position_select . '
                  </div>
                  <div style="margin-bottom: 10px;">
                    <label style="display: block; margin-bottom: 5px;">' . esc_html__('Logo Size (%):', 'khattam') . '</label>
                    ' . $size_input . '
                  </div>
                  <div style="margin-bottom: 10px;">
                    <label style="display: block; margin-bottom: 5px;">' . esc_html__('Logo Opacity (%):', 'khattam') . '</label>
                    ' . $opacity_input . '
                  </div>
                  <button type="button" class="button khattam-apply-watermark" data-id="' . $post->ID . '">' . esc_html__('Apply Watermark', 'khattam') . '</button>'
    );
    
    $form_fields['khattam_watermark']['html'] .= '<button type="button" class="button khattam-remove-watermark" data-id="' . $post->ID . '">' . esc_html__('Remove Watermark', 'khattam') . '</button>';
    return $form_fields;
}
add_filter('attachment_fields_to_edit', 'khattam_add_watermark_button', 10, 2);

// Update watermark settings when saving image details
function khattam_save_attachment_fields($post, $attachment) {
    if (isset($attachment['khattam_logo_position'])) {
        update_post_meta($post['ID'], '_khattam_logo_position', sanitize_text_field($attachment['khattam_logo_position']));
    }
    if (isset($attachment['khattam_logo_size'])) {
        update_post_meta($post['ID'], '_khattam_logo_size', intval($attachment['khattam_logo_size']));
    }
    if (isset($attachment['khattam_logo_opacity'])) {
        update_post_meta($post['ID'], '_khattam_logo_opacity', intval($attachment['khattam_logo_opacity']));
    }
    return $post;
}
add_filter('attachment_fields_to_save', 'khattam_save_attachment_fields', 10, 2);

// Delete original image when main image is deleted
function khattam_delete_original_image($post_id) {
    if (wp_attachment_is_image($post_id)) {
        $original_file = get_post_meta($post_id, '_khattam_original_file', true);
        if ($original_file && file_exists($original_file)) {
            // Replace error_log with proper logging
            if (WP_DEBUG) {
                $message = sprintf('Khattam: %s %s', 
                    esc_html__('Deleting original file:', 'khattam'),
                    $original_file
                );
                error_log($message);
            }
            wp_delete_file($original_file);
        }
        delete_post_meta($post_id, '_khattam_original_file');
    }
}
add_action('delete_attachment', 'khattam_delete_original_image');

// AJAX handler for watermark removal
function khattam_ajax_remove_watermark() {
    check_ajax_referer('khattam_watermark_nonce', 'nonce');
    
    if (!current_user_can('upload_files')) {
        wp_send_json_error(esc_html__('Unauthorized', 'khattam'));
    }
    
    // Validate and sanitize image_id
    if (!isset($_POST['image_id'])) {
        wp_send_json_error(esc_html__('Image ID is required', 'khattam'));
    }
    $image_id = intval(wp_unslash($_POST['image_id']));
    
    $original_file = get_post_meta($image_id, '_khattam_original_file', true);
    
    if (empty($original_file) || !file_exists($original_file)) {
        wp_send_json_error(esc_html__('Original image not found', 'khattam'));
    }
    
    $current_file = get_attached_file($image_id);
    
    if (copy($original_file, $current_file)) {
        wp_delete_file($original_file);
        delete_post_meta($image_id, '_khattam_original_file');
        wp_update_attachment_metadata($image_id, wp_generate_attachment_metadata($image_id, $current_file));
        wp_send_json_success(array('url' => wp_get_attachment_url($image_id)));
    } else {
        wp_send_json_error(esc_html__('Failed to restore original image', 'khattam'));
    }
}
add_action('wp_ajax_khattam_remove_watermark', 'khattam_ajax_remove_watermark');

// Add bulk action buttons
function khattam_add_bulk_actions($bulk_actions) {
    // Check if button display option is enabled
    if (get_option('khattam_show_media_buttons', '1') === '1') {
        $bulk_actions['apply_watermark'] = esc_html__('Apply Watermark', 'khattam');
        $bulk_actions['remove_watermark'] = esc_html__('Remove Watermark', 'khattam');
    }
    return $bulk_actions;
}
add_filter('bulk_actions-upload', 'khattam_add_bulk_actions');

// Handle bulk actions
function khattam_handle_bulk_actions($redirect_to, $doaction, $post_ids) {
    if ($doaction !== 'apply_watermark' && $doaction !== 'remove_watermark') {
        return $redirect_to;
    }

    // Store operation ID and data in WordPress options
    $operation_id = uniqid('khattam_bulk_');
    $operation_data = array(
        'action' => $doaction,
        'total' => count($post_ids),
        'processed' => 0,
        'post_ids' => $post_ids,
        'status' => 'pending'
    );
    update_option($operation_id, $operation_data);

    // Add operation ID to redirect URL
    $redirect_to = add_query_arg(array(
        'bulk_watermark_operation' => $operation_id
    ), $redirect_to);

    return $redirect_to;
}
add_filter('handle_bulk_actions-upload', 'khattam_handle_bulk_actions', 10, 3);

// Add success message for bulk actions
function khattam_bulk_action_admin_notice() {
    if (!empty($_REQUEST['bulk_watermark_processed'])) {
        $processed_count = intval($_REQUEST['bulk_watermark_processed']);
        
        /* translators: %s: number of processed images */
        printf(
            '<div class="notice notice-success is-dismissible"><p>' .
            wp_kses(
                _n(
                    '%s image processed successfully.',
                    '%s images processed successfully.',
                    $processed_count,
                    'khattam'
                ),
                array('strong' => array())
            ) . '</p></div>',
            esc_html($processed_count)
        );
    }
}
add_action('admin_notices', 'khattam_bulk_action_admin_notice');

// Internal function to remove watermark for all sizes
function khattam_remove_watermark_internal($image_id) {
    $original_file = get_post_meta($image_id, '_khattam_original_file', true);
    
    if (empty($original_file) || !file_exists($original_file)) {
        return false;
    }
    
    $current_file = get_attached_file($image_id);
    $current_file = wp_normalize_path($current_file);
    
    if (copy($original_file, $current_file)) {
        wp_delete_file($original_file);
        delete_post_meta($image_id, '_khattam_original_file');
        
        // Restore all image sizes
        $metadata = wp_get_attachment_metadata($image_id);
        $dir = pathinfo($current_file, PATHINFO_DIRNAME);
        
        if (isset($metadata['sizes']) && is_array($metadata['sizes'])) {
            foreach ($metadata['sizes'] as $size => $size_info) {
                $size_file = $dir . '/' . $size_info['file'];
                $size_original_file = $dir . '/' . pathinfo($size_info['file'], PATHINFO_FILENAME) . '_original.' . pathinfo($size_info['file'], PATHINFO_EXTENSION);
                
                if (file_exists($size_original_file)) {
                    copy($size_original_file, $size_file);
                    wp_delete_file($size_original_file);
                }
            }
        }
        
        wp_update_attachment_metadata($image_id, wp_generate_attachment_metadata($image_id, $current_file));
        return true;
    }
    
    return false;
}

function khattam_enqueue_editor_assets() {
    wp_enqueue_script(
        'khattam-logo-image-overlay',
        plugins_url('includes/js/logo-image-overlay.js', __FILE__),
        array('wp-plugins', 'wp-edit-post', 'wp-i18n', 'wp-element', 'wp-components', 'wp-data', 'wp-block-editor'),
        filemtime(plugin_dir_path(__FILE__) . 'includes/js/logo-image-overlay.js'),
        true
    );

    wp_localize_script('khattam-logo-image-overlay', 'khattamVars', array(
        'nonce' => wp_create_nonce('khattam_watermark_nonce'),
        'logoPosition' => get_option('khattam_logo_position', 'bottom-right'),
        'logoSize' => get_option('khattam_logo_size', 10),
        'logoOpacity' => get_option('khattam_logo_opacity', 100)
    ));
}
add_action('enqueue_block_editor_assets', 'khattam_enqueue_editor_assets');

// Add settings link to plugins page
function khattam_add_settings_link($links) {
    $settings_link = '<a href="' . admin_url('options-general.php?page=khattam') . '">' . 
        esc_html__('Settings', 'khattam') . '</a>';
    array_unshift($links, $settings_link);
    return $links;
}

$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 'khattam_add_settings_link');

function khattam_localize_scripts() {
    wp_localize_script('khattam-watermark-dialog', 'khattamVars', array(
        'nonce' => wp_create_nonce('khattam_watermark_nonce'),
        'showMediaButtons' => get_option('khattam_show_media_buttons', '1'),
        'i18n' => array(
            'addWatermark' => __('Add Watermark', 'khattam'),
            'confirmAddLogo' => __('Do you want to add the logo to this image?', 'khattam'),
            'yes' => __('Yes', 'khattam'),
            'no' => __('No', 'khattam'),
            'logoAddedSuccess' => __('Logo added successfully', 'khattam'),
            'serverError' => __('Server connection error', 'khattam'),
            'errorAddingLogo' => __('Error adding logo', 'khattam'),
            'applying' => __('Applying...', 'khattam'),
            'removing' => __('Removing...', 'khattam'),
            'confirmRemove' => __('Are you sure you want to remove the logo?', 'khattam'),
            'logoRemovedSuccess' => __('Logo removed successfully', 'khattam'),
            'errorRemovingLogo' => __('Error removing logo', 'khattam'),
            'chooseOrUploadLogo' => __('Choose or Upload Logo', 'khattam'),
            'useThisLogo' => __('Use this Logo', 'khattam'),
            'applyWatermark' => __('Apply Watermark', 'khattam'),
            'removeWatermark' => __('Remove Watermark', 'khattam'),
            'unknownError' => __('Unknown error', 'khattam'),
            'noImagesFound' => __('No images found in this post', 'khattam'),
            'logoAppliedToAllImages' => __('Logo applied successfully to all images', 'khattam'),
            'errorApplyingLogo' => __('Error applying logo to images:', 'khattam'),
            'errorApplyingSomeLogos' => __('Error applying logo to some images', 'khattam'),
            'logoRemovedFromAllImages' => __('Logo removed successfully from all images', 'khattam'),
            'errorRemovingSomeLogos' => __('Error removing logo from some images', 'khattam'),
            'manageLogo' => __('Manage Logo on Images', 'khattam'),
            'logoPosition' => __('Logo Position', 'khattam'),
            'topLeft' => __('Top Left', 'khattam'),
            'topRight' => __('Top Right', 'khattam'),
            'bottomLeft' => __('Bottom Left', 'khattam'),
            'bottomRight' => __('Bottom Right', 'khattam'),
            'logoSizePercent' => __('Logo Size (%)', 'khattam'),
            'logoOpacityPercent' => __('Logo Opacity (%)', 'khattam'),
            'applyingLogo' => __('Applying logo...', 'khattam'),
            'applyLogoToAllImages' => __('Apply Logo to All Images', 'khattam'),
            'removingLogo' => __('Removing logo...', 'khattam'),
            'removeLogoFromAllImages' => __('Remove Logo from All Images', 'khattam'),
            'numberOfImages' => __('Number of images in post', 'khattam'),
            'addWatermark' => __('Add Watermark', 'khattam'),
            'confirmAddLogo' => __('Do you want to add the logo to this image?', 'khattam'),
            'yes' => __('Yes', 'khattam'),
            'no' => __('No', 'khattam'),
            'logoAddedSuccess' => __('Logo added successfully', 'khattam'),
            'serverError' => __('Server connection error', 'khattam'),
            'errorAddingLogo' => __('Error adding logo', 'khattam'),
            'processingImage' => __('Processing image...', 'khattam'),
            'dialogTitle' => __('Watermark Settings', 'khattam'),
            'closeDialog' => __('Close', 'khattam'),
            'confirmCleanup' => __('Are you sure you want to delete ALL original images? This action is irreversible!', 'khattam'),
            'cleanupInProgress' => __('Deleting Original Images...', 'khattam'),
            'cleanupComplete' => __('Cleanup Completed!', 'khattam'),
            'cleanupError' => __('Error during cleanup process', 'khattam'),
            'deletedFiles' => __('Files deleted', 'khattam'),
            'totalDeleted' => __('Total files deleted', 'khattam'),
            'cancel' => __('Cancel', 'khattam'),
            'close' => __('Close', 'khattam')
        )
    ));
}
add_action('admin_enqueue_scripts', 'khattam_localize_scripts');

// Add bulk operation screen
function khattam_bulk_operation_screen() {
    $screen = get_current_screen();
    if ($screen->id !== 'upload') {
        return;
    }

    if (!isset($_GET['bulk_watermark_operation'])) {
        return;
    }
    
    $operation_id = sanitize_text_field(wp_unslash($_GET['bulk_watermark_operation']));
    $operation_data = get_option($operation_id);

    if (!$operation_data) {
        return;
    }

    // Display HTML template only
    ?>
    <div id="khattam-bulk-progress" class="notice notice-info">
        <h2><?php echo esc_html($operation_data['action'] === 'apply_watermark' ? 
            __('Applying Watermark...', 'khattam') : 
            __('Removing Watermark...', 'khattam')); ?></h2>
        <div class="progress-bar-container">
            <div class="progress-bar"></div>
        </div>
        <div class="progress-text">0%</div>
        <div class="operation-controls">
            <button class="cancel-operation button button-secondary">
                <?php esc_html_e('Cancel Operation', 'khattam'); ?>
            </button>
        </div>
    </div>
    <?php
}
add_action('admin_notices', 'khattam_bulk_operation_screen');

// Add AJAX handler for batch processing
function khattam_process_bulk_batch() {
    check_ajax_referer('khattam_bulk_operation', 'nonce');
    
    if (!current_user_can('upload_files')) {
        wp_send_json_error(array('message' => 'Unauthorized'));
    }

    // Validate and sanitize operation_id
    if (!isset($_POST['operation_id'])) {
        wp_send_json_error(array('message' => 'Operation ID is required'));
    }
    $operation_id = sanitize_text_field(wp_unslash($_POST['operation_id']));
    
    $operation_data = get_option($operation_id);

    if (!$operation_data) {
        wp_send_json_error(array('message' => 'Operation not found'));
    }

    // Process batch of 5 images at a time
    $batch_size = 5;
    $remaining_ids = array_slice($operation_data['post_ids'], $operation_data['processed'], $batch_size);

    foreach ($remaining_ids as $post_id) {
        if ($operation_data['action'] === 'apply_watermark') {
            // Apply watermark
            $file = get_attached_file($post_id);
            $file_array = array('file' => $file, 'attachment_id' => $post_id);
            khattam_add_watermark($file_array);
        } else {
            // Remove watermark
            khattam_remove_watermark_internal($post_id);
        }
        $operation_data['processed']++;
    }

    // Update operation status
    if ($operation_data['processed'] >= $operation_data['total']) {
        $operation_data['status'] = 'completed';
    }
    update_option($operation_id, $operation_data);

    wp_send_json_success($operation_data);
}
add_action('wp_ajax_khattam_process_bulk_batch', 'khattam_process_bulk_batch');

// Add AJAX handler for operation cancellation
function khattam_cancel_bulk_operation() {
    check_ajax_referer('khattam_bulk_operation', 'nonce');
    
    if (!current_user_can('upload_files')) {
        wp_send_json_error(array('message' => 'Unauthorized'));
    }

    // Validate and sanitize operation_id
    if (!isset($_POST['operation_id'])) {
        wp_send_json_error(array('message' => 'Operation ID is required'));
    }
    $operation_id = sanitize_text_field(wp_unslash($_POST['operation_id']));
    
    delete_option($operation_id);
    wp_send_json_success();
}
add_action('wp_ajax_khattam_cancel_bulk_operation', 'khattam_cancel_bulk_operation');

function khattam_admin_enqueue_scripts($hook) {
    // Load styles and scripts only on required pages
    if (!in_array($hook, array('upload.php', 'post.php', 'post-new.php', 'settings_page_khattam'))) {
        return;
    }

    // Register and enqueue CSS
    wp_register_style(
        'khattam-admin-style',
        plugins_url('includes/css/admin.css', __FILE__),
        array(),
        filemtime(plugin_dir_path(__FILE__) . 'includes/css/admin.css')
    );
    wp_enqueue_style('khattam-admin-style');

    // Register and enqueue scripts
    wp_register_script(
        'khattam-bulk-operations',
        plugins_url('includes/js/bulk-operations.js', __FILE__),
        array('jquery'),
        filemtime(plugin_dir_path(__FILE__) . 'includes/js/bulk-operations.js'),
        true
    );

    // Add variables to the script
    wp_localize_script('khattam-bulk-operations', 'khattamBulkVars', array(
        'nonce' => wp_create_nonce('khattam_bulk_operation'),
        'adminUrl' => admin_url('upload.php'),
        'i18n' => array(
            'confirmCancel' => esc_html__('Are you sure you want to cancel the operation?', 'khattam'),
            'operationCompleted' => esc_html__('Operation completed successfully!', 'khattam'),
            'returnToMedia' => esc_html__('Return to Media Library', 'khattam'),
            'connectionError' => esc_html__('Connection error occurred', 'khattam')
        )
    ));

    wp_enqueue_script('khattam-bulk-operations');
}
add_action('admin_enqueue_scripts', 'khattam_admin_enqueue_scripts');

// AJAX handler for cleanup process
function khattam_ajax_cleanup_originals() {
    check_ajax_referer('khattam_watermark_nonce', 'nonce');
    
    if (!current_user_can('manage_options')) {
        wp_send_json_error(esc_html__('Unauthorized', 'khattam'));
    }
    
    $batch_size = isset($_POST['batch_size']) ? intval($_POST['batch_size']) : 50;
    $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
    
    $args = array(
        'post_type'      => 'attachment',
        'post_mime_type' => 'image',
        'post_status'    => 'inherit',
        'posts_per_page' => $batch_size,
        'paged'          => $page,
        'meta_query'     => array(
            array(
                'key'     => '_khattam_original_file',
                'compare' => 'EXISTS'
            )
        ),
        'fields' => 'ids'
    );
    
    $query = new WP_Query($args);
    $total = $query->found_posts;
    $processed = 0;
    
    foreach ($query->posts as $post_id) {
        $original_file = get_post_meta($post_id, '_khattam_original_file', true);
        if ($original_file && file_exists($original_file)) {
            if (wp_delete_file($original_file)) {
                delete_post_meta($post_id, '_khattam_original_file');
                $processed++;
            }
        }
    }
    
    wp_send_json_success(array(
        'total'     => $total,
        'processed' => ($page - 1) * $batch_size + $processed,
        'remaining' => max(0, $total - ($page * $batch_size)),
        'page'      => $page + 1,
        'complete'  => ($page * $batch_size) >= $total
    ));
}
add_action('wp_ajax_khattam_cleanup_originals', 'khattam_ajax_cleanup_originals');
